﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace TrackShapeLayerTest
{
    public partial class Form1 : Form
    {
        private InMemoryFeatureLayer featureLayer = new InMemoryFeatureLayer();
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var vertices = new Collection<Vertex>();
            vertices.Add(new Vertex(12, 35));
            vertices.Add(new Vertex(11, 32));
            vertices.Add(new Vertex(13, 38));
            vertices.Add(new Vertex(20, 22));
            vertices.Add(new Vertex(25, 45));

            var polygon = new RectangleShape(35, 80, 55, 60);

            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(new Feature(new PointShape(30, 40), new Dictionary<string, string>() { { "id", "1" } }));
            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(new Feature(new PointShape(10, 40), new Dictionary<string, string>() { { "id", "2" } }));
            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(new Feature(new LineShape(vertices), new Dictionary<string, string>() { { "id", "3" } }));
            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(new Feature(polygon, new Dictionary<string, string>() { { "id", "4" } }));
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = null;
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = null;
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = null;

            var valueStyle1 = new ValueStyle();
            valueStyle1.ColumnName = "id";
            valueStyle1.ValueItems.Add(new ValueItem("4", new AreaStyle(new GeoSolidBrush(GeoColor.SimpleColors.Blue))));
            valueStyle1.ValueItems.Add(new ValueItem("3", new LineStyle(new GeoPen(GeoColor.SimpleColors.Blue, 2.0f))));
            valueStyle1.ValueItems.Add(new ValueItem("2", new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.SimpleColors.Blue), 20)));
            valueStyle1.ValueItems.Add(new ValueItem("1", new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.SimpleColors.Blue), 20)));

            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(valueStyle1);
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            winformsMap1.TrackOverlay.MapMouseClick += TrackOverlay_MapMouseClick;
            var overlay = new LayerOverlay();
            overlay.Layers.Add(featureLayer);

            featureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(new GeoSolidBrush(GeoColor.SimpleColors.Red), 4.0f));
            featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            winformsMap1.Overlays.Add(overlay);
            winformsMap1.CurrentExtent = new RectangleShape(0, 100, 60, 0);
            winformsMap1.Refresh();
        }

        private void TrackOverlay_MapMouseClick(object sender, MapMouseClickInteractiveOverlayEventArgs e)
        {
            var features = winformsMap1.TrackOverlay.TrackShapeLayer.FeatureSource.GetFeaturesNearestTo(new Feature(new PointShape(e.InteractionArguments.WorldX, e.InteractionArguments.WorldY)), GeographyUnit.DecimalDegree, 1, ReturningColumnsType.AllColumns);
            featureLayer.InternalFeatures.Clear();

            var feature = new Feature(features[0].GetBoundingBox().Buffer(100, GeographyUnit.DecimalDegree, DistanceUnit.Kilometer).GetBoundingBox());

            var valueStyle = new ValueStyle();
            valueStyle.ColumnName = "id";

            if (features[0].GetShape() is AreaBaseShape)
                valueStyle.ValueItems.Add(new ValueItem(features[0].ColumnValues["id"], new AreaStyle(new GeoSolidBrush(GeoColor.SimpleColors.Red))));

            if(features[0].GetShape() is PointShape)
                valueStyle.ValueItems.Add(new ValueItem(features[0].ColumnValues["id"], new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.SimpleColors.Red), 20)));

            if (features[0].GetShape() is LineShape)
                valueStyle.ValueItems.Add(new ValueItem(features[0].ColumnValues["id"], new LineStyle(new GeoPen(GeoColor.SimpleColors.Red, 2.0f))));

            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(valueStyle);
            winformsMap1.TrackOverlay.TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            featureLayer.InternalFeatures.Add(feature);

            winformsMap1.Refresh();
        }
    }
}
